"""
Dolphin MCP - A flexible Python client for interacting with Model Context Protocol (MCP) servers.
"""

from .client import MCPClient, run_interaction

__version__ = "0.1.2"
__all__ = ["MCPClient", "run_interaction"]
